/*$Id: u_parameter.h,v 25.60 2005/11/30 05:52:21 al Exp $ -*- C++ -*-
 * Copyright (C) 2005 Albert Davis
 * Author: Albert Davis <aldavis@ieee.org>
 *
 * This file is part of "Gnucap", the Gnu Circuit Analysis Package
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *------------------------------------------------------------------
 * A class for parameterized values
 * Used for spice compatible .param statements
 * and passing arguments to models and subcircuits
 */
//testing=obsolete
#ifndef U_PARAMETER_H
#define U_PARAMETER_H
#include "io_.h"
#include "constant.h"
#include "ap.h"
/*--------------------------------------------------------------------------*/
class CS;
class OMSTREAM;
double parameter_lookup(std::string& s);
/*--------------------------------------------------------------------------*/
template <class T>
class PARAMETER {
public:
  T _v;
  std::string _s;
  bool _hit;
public:
  explicit PARAMETER() :_v(NOT_INPUT), _s(), _hit(false) {}
  PARAMETER(const PARAMETER<double>& p) :_v(p._v), _s(p._s), _hit(false) {}
  explicit PARAMETER(T v) :_v(v), _s() {}
  explicit PARAMETER(T v, const std::string& s) :_v(v), _s(s), _hit(false) {}
  
  operator T()const {return _v;}
  bool	has_value()const {return (_s != "");}
  T	e_val(T def);
  void	parse(CS& cmd);
  void	print(OMSTREAM& o)const {
    if (_s == "#") {
      o << _v;
    }else if (_s == "") {
      o << "NA(" << _v << ")";
    }else{
      o << _s;
    }
  }
  void	operator=(T v)	 {_v = v; _s = "#";}
  T*	pointer_hack()	 {return &_v;}
};
/*--------------------------------------------------------------------------*/
class PARAM_LIST {
public:
  std::map<const std::string, PARAMETER<double> > _pl;
public:
  typedef std::map<const std::string, PARAMETER<double> >::const_iterator
		const_iterator;
  void	parse(CS& cmd);
  void	print(OMSTREAM&)const;
};
/*--------------------------------------------------------------------------*/
template <class T>
inline T PARAMETER<T>::e_val(T def) {
  _hit = true;
  if (_s == "") {
    _v = def;
  }else if (_s != "#") {
    _v = static_cast<T>(parameter_lookup(_s));
  }else{
    //assert(_v != NOT_INPUT);
  }
  return _v;
}
/*--------------------------------------------------------------------------*/
template <>
inline void PARAMETER<bool>::parse(CS& cmd) {
  if (!cmd.match1('_') || cmd.is_float()) {
    cmd >> _v; _s = "#";
  }else{
    cmd >> _s;
  }
}

template <>
inline void PARAMETER<int>::parse(CS& cmd) {
  if (!cmd.match1('_') || cmd.is_float()) {
    cmd >> _v; _s = "#";
  }else{
    cmd >> _s;
  }
}

template <>
inline void PARAMETER<double>::parse(CS& cmd) {
  if (!cmd.match1('_') || cmd.is_float()) {
    cmd >> _v; _s = "#";
  }else{
    cmd >> _s;
  }
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
bool get(CS& cmd, const std::string& key, PARAMETER<bool>* val);
bool get(CS& cmd, const std::string& key, PARAMETER<int>* val);
/*--------------------------------------------------------------------------*/
template <class T>
bool get(CS& cmd, const std::string& key, PARAMETER<T>* val, AP_MOD mod)
{
  return get(cmd, key, &(val->_v), mod);
}
/*--------------------------------------------------------------------------*/
template <class T>
bool get(CS& cmd, const std::string& key,PARAMETER<T>* val,AP_MOD mod,T scale)
{
  return get(cmd, key, &(val->_v), mod, scale);
}
/*--------------------------------------------------------------------------*/
template <class T>
inline OMSTREAM& operator<<(OMSTREAM& o, const PARAMETER<T> p)
{
  p.print(o);
  return o;
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
#endif
